//
//  ComposeReplyView.swift
//  Tusker
//
//  Created by Shadowfacts on 8/22/20.
//  Copyright © 2020 Shadowfacts. All rights reserved.
//

import SwiftUI

struct ComposeReplyView: View {
    let status: StatusMO
    let stackPadding: CGFloat
    let outerMinY: CGFloat
    
    @State private var contentHeight: CGFloat?
    
    @ObservedObject private var preferences = Preferences.shared
    
    private let horizSpacing: CGFloat = 8
    
    var body: some View {
        HStack(alignment: .top, spacing: horizSpacing) {
            GeometryReader(content: self.replyAvatarImage)
                .frame(width: 50)

            VStack(alignment: .leading, spacing: 0) {
                HStack {
                    AccountDisplayNameLabel(account: status.account, fontSize: 17)
                        .lineLimit(1)
                        .layoutPriority(1)

                    Text(verbatim: "@\(status.account.acct)")
                        .font(.system(size: 17, weight: .light))
                        .foregroundColor(.secondary)
                        .lineLimit(1)

                    Spacer()
                }

                ComposeReplyContentView(status: status) { (newHeight) in
                    self.contentHeight = newHeight
                }
                    .offset(x: -4, y: -8)
                    .padding(.bottom, -8)
            }
            .frame(height: max(50, contentHeight ?? 0) + 8)
        }
        .padding(.bottom, -8)
    }
    
    private func replyAvatarImage(geometry: GeometryProxy) -> some View {
        // using named coordinate spaces produces an incorrect scroll offset on iOS 13,
        // so simply compare the geometry inside and outside the scroll view in the global coordinate space
        let scrollOffset = outerMinY - geometry.frame(in: .global).minY
        
        // add stackPadding so that the image is always at least stackPadding away from the top
        var offset = scrollOffset + stackPadding
        
        // offset can never be less than 0 (i.e., above the top of the in-reply-to content)
        offset = max(offset, 0)
        
        // subtract 50, because we care about where the bottom of the view is but the offset is relative to the top of the view
        let maxOffset = (contentHeight ?? 0) - 50
        
        // once you scroll past the in-reply-to-content, the bottom of the avatar should be pinned to the bottom of the content
        offset = min(offset, maxOffset)
        
        return ComposeAvatarImageView(url: status.account.avatar)
            .frame(width: 50, height: 50)
            .cornerRadius(preferences.avatarStyle.cornerRadiusFraction * 50)
            .offset(x: 0, y: offset)
    }
    
}

//struct ComposeReplyView_Previews: PreviewProvider {
//    static var previews: some View {
//        ComposeReplyView()
//    }
//}
